using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Net;

namespace PocketRassegnaRAI
{
    /// <summary>
    /// Classe per la gestione del download di un file
    /// </summary>
    public class CFileDownloader
    {
        #region  Private fields

        private string _strFileURL = string.Empty;
        private string _strFilePath = string.Empty;
        //grandezza totale del file
        private long _lngFileSize = 0;
        //byte scaricati del file
        private long _lngFileDownloadedSize = 0;
        //percentuale del file scaricata 
        private bool _booIsDownloading = false;
        private bool _booIsDownloadSuccessful = false;
        private bool _booIsStarted = false;
        private bool _booCancelDownload = false;
        private string _strProxyURI = string.Empty;
        private int _intBufferMaxSize = 32000;

        #endregion

        #region Public fields

        public string FileURL
        {
            get { return _strFileURL; }
            set { _strFileURL = value; }
        }

        public string FilePath
        {
            get { return _strFilePath; }
            set {
                if (value.EndsWith("\\")){
                    _strFilePath = value + FileName;
                }else{
                    _strFilePath = value;
                }
            }
        }

        public string FileName
        {
            get
            {
                string strFileName;

                if (_strFileURL.Equals(string.Empty))
                {
                    throw new ArgumentException("Please supply a document url.");
                }
                int loc = _strFileURL.LastIndexOf("/") + 1;
                int len = _strFileURL.Length - loc;
                strFileName = _strFileURL.Substring(loc, len);
                return strFileName;
            }
        }

        public long FileSize
        {
            get { return _lngFileSize; }
            set { _lngFileSize = value; }
        }

        public long FileDownloadedSize
        {
            get { return _lngFileDownloadedSize; }
            set { _lngFileDownloadedSize = value; }
        }

        public bool IsDownloading
        {
            get { return _booIsDownloading; }
            set { _booIsDownloading = value; }
        }

        public bool IsDownloadSuccessful
        {
            get { return _booIsDownloadSuccessful; }
            set { _booIsDownloadSuccessful = value; }
        }

        public bool IsStarted
        {
            get { return _booIsStarted; }
            set { _booIsStarted = value; }
        }

        public string ProxyURI
        {
            get { return _strProxyURI; }
            set { _strProxyURI = value; }
        }

        public int BufferMaxSize
        {
            get { return _intBufferMaxSize; }
            set { _intBufferMaxSize = value; }
        }

        #endregion

        #region Delegates

        public delegate void delDownloadStarting(CFileDownloader objThread);
        public delegate void delDownloadInProgress(CFileDownloader objThread);
        public delegate void delDownloadCompleted(CFileDownloader objThread, bool booIsSuccess, string strErrorMessage);

        #endregion

        #region Events
        public event delDownloadStarting DownloadStarting;
        public event delDownloadInProgress DownloadInProgress;
        public event delDownloadCompleted DownloadCompleted;

        #endregion
        
        #region Private methods


        #endregion

        #region Public methods

        public CFileDownloader(string strFileURL, string strFilePath)
        {
            _strFileURL = strFileURL;
            FilePath = strFilePath;
        }

        /// <summary>
        /// Starts the download of the attached url into the given directory.
        /// </summary>
        public void StartDownload()
        {
            HttpWebResponse objResponse = null;
            IWebProxy objProxy = null;
            Stream objStream = null;
            FileStream objFileStream = null;
            string strDestFileName;
            string strError = String.Empty;
            Byte[] arrBuffer;
            int intBytesRead = 0;


            //if (_strFileURL.Equals(string.Empty))
            //{
            //    throw new ArgumentException("Please supply a document url.");
            //}
            //if (_strFilePath.Equals(string.Empty))
            //{
            //    throw new ArgumentException("Please supply a directory.");
            //}
            _booIsStarted = true;
            
            //raise the DownloadStarting event
            DownloadStarting(this);

            _booIsDownloading = true;
            _booIsDownloadSuccessful = false;
            _booCancelDownload = false;

            try
            {
                _booIsDownloadSuccessful = false;
                strDestFileName = _strFilePath.Replace("/", " ").Replace("%20", " ");

                if (_strProxyURI != null && _strProxyURI != string.Empty)
                {
                    objProxy = new WebProxy(_strProxyURI, true);
                    objProxy.Credentials = System.Net.CredentialCache.DefaultCredentials;
                }
                HttpWebRequest request = (HttpWebRequest)WebRequest.Create(_strFileURL);
                if (objProxy != null)
                {
                    request.Proxy = objProxy;
                }

                objResponse = (HttpWebResponse)request.GetResponse();
                objStream = objResponse.GetResponseStream();

                _lngFileSize = objResponse.ContentLength;
                arrBuffer= new Byte[_intBufferMaxSize];
                objFileStream = new FileStream(strDestFileName, FileMode.OpenOrCreate, FileAccess.Write);

                do
            	{
                    //controlla se e' stato cancellato il download
                    if (_booCancelDownload){
                        break;
                    }

                    //legge un pezzo dello stream
                    intBytesRead = objResponse.GetResponseStream().Read(arrBuffer, 0, _intBufferMaxSize);
                    _lngFileDownloadedSize += intBytesRead;

                    if (0 == intBytesRead)
                    {
                        _booIsDownloadSuccessful = true;
                        //eventualmente da togliere
                        objStream.Close();
                        break;
                    }
                    objFileStream.Write(arrBuffer, 0, intBytesRead);

                    DownloadInProgress(this);
                } while (true);
                //objStream.Close();
                objFileStream.Close();
            }
            catch (Exception objEx)
            {
                strError = objEx.Message;
            }
            finally
            {
                _booIsDownloading = false;
                //raise the DownloadComplete event
                DownloadCompleted(this, _booIsDownloadSuccessful, strError);
            }
        }

        public void CancelDownload()
        {
            _booCancelDownload = true;
        }

        #endregion
    }
}
