using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Collections;

namespace PocketRassegnaRAI
{
    public partial class frmFileSelect : Form
    {
        private string _strMask;
        public string FileName;

        public frmFileSelect(string strMask)
		{
			_strMask = strMask;
			DialogResult = DialogResult.Cancel;
            init();
        }

		public frmFileSelect()
		{
			_strMask = "*.*";
            DialogResult = DialogResult.Cancel;
            init();
		}
        
        private void init()
        {
            int intHeight;

            InitializeComponent();

            intHeight = this.ClientSize.Height / 2;
            treeView1.Height = intHeight;
            listView1.Top = intHeight;
            listView1.Height = intHeight;

            FillTree();
            menuItemOk.Enabled = false;
        }

        private void ExpandNode(TreeNodeCollection objParentNode, string strRootDir)
        {
            DirectoryInfo objDI;
            DirectoryInfo[] colDirs;

            objDI = new DirectoryInfo(strRootDir);
            colDirs = objDI.GetDirectories();
            // Sort Dirs
            ArrayList arrDirSorted = new ArrayList(colDirs.Length);
            foreach (DirectoryInfo objDI1 in colDirs)
            {
                arrDirSorted.Add(objDI1);
            }
            arrDirSorted.Sort(new myDirSort());

            foreach (DirectoryInfo objDI2 in arrDirSorted)
            {
                TreeNode objTn = new TreeNode(objDI2.Name);
                objTn.Tag = objDI2.FullName;
                objTn.ImageIndex = 0;
                objTn.SelectedImageIndex = 1;
                objParentNode.Add(objTn);
                ExpandNode(objTn.Nodes, objDI2.FullName);
            }
        }

        private void FillTree()
        {
            treeView1.Nodes.Clear();
            Cursor.Current = Cursors.WaitCursor;
            ExpandNode(treeView1.Nodes, @"\");
            Cursor.Current = Cursors.Default;
            treeView1.SelectedNode = treeView1.Nodes[0];
        }


		private void treeView1_AfterSelect(object sender, System.Windows.Forms.TreeViewEventArgs e)
		{
            DirectoryInfo objDI;
            FileInfo[] arrFiles;
            ArrayList arrFileSorted;
            ListViewItem objItem;
            double dblLen;
            string strSuffix;

			listView1.Items.Clear();
			objDI = new DirectoryInfo((string)e.Node.Tag);
			arrFiles = objDI.GetFiles(_strMask);
			arrFileSorted = new ArrayList(arrFiles.Length);
			foreach(FileInfo objFI1 in arrFiles)
			{
				arrFileSorted.Add(objFI1);
			}
			arrFileSorted.Sort(new myFileSort());

			foreach(FileInfo objFI2 in arrFileSorted)
			{
				objItem = new ListViewItem(objFI2.Name);
				dblLen = (double)objFI2.Length;
				strSuffix = "b";
				if (dblLen > (double)1024.0)
				{
					dblLen/=(double)1024.0;
					strSuffix="Kb";
				}
				if (dblLen > (double)1024.0)
				{
					dblLen/=(double)1024.0;
					strSuffix="Mb";
				}				
				objItem.SubItems.Add(dblLen.ToString("0.##") + strSuffix);
				objItem.SubItems.Add(objFI2.CreationTime.ToString());
				listView1.Items.Add(objItem);
			}
		}

		private void listView1_ItemActivate(object sender, System.EventArgs e)
		{
			menuItemOk.Enabled = true;
		}

		private void listView1_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			if (1 == listView1.SelectedIndices.Count)
                menuItemOk.Enabled = true;
			else
                menuItemOk.Enabled = false;
		}

        private void menuItemOk_Click(object sender, EventArgs e)
        {
            if (1 != listView1.SelectedIndices.Count)
                return;

            FileName = treeView1.SelectedNode.Tag + @"\" + listView1.Items[listView1.SelectedIndices[0]].Text;
            DialogResult = DialogResult.OK;
            Close();
        }

        private void menuItemCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void frmFileSelect_Resize(object sender, EventArgs e)
        {
            int intHeight;

            intHeight = this.ClientSize.Height / 2;
            treeView1.Size = new Size(this.ClientSize.Width, Height = intHeight);
            listView1.Size = new Size(this.ClientSize.Width, Height = intHeight);
            listView1.Top = intHeight;
        }
	}

	public class myDirSort : IComparer  
	{
		// Calls CaseInsensitiveComparer.Compare with the parameters reversed.
		int IComparer.Compare(Object ax, Object ay )  
		{
			DirectoryInfo x = (DirectoryInfo)ax;
			DirectoryInfo y = (DirectoryInfo)ay;
			return x.Name.CompareTo(y.Name);
		}
	}
	
    public class myFileSort : IComparer  
	{
		// Calls CaseInsensitiveComparer.Compare with the parameters reversed.
		int IComparer.Compare(Object ax, Object ay )  
		{
			FileInfo x = (FileInfo)ax;
			FileInfo y = (FileInfo)ay;
			return x.Name.CompareTo(y.Name);
		}
	}
}